using Microsoft.AspNetCore.Authentication;
using Microsoft.IdentityModel.Protocols.OpenIdConnect;

namespace ReactAndOpenApi.Endpoints;

public static class BackendForFrontendEndpoint
{
    public static WebApplication MapBackendForFrontendEndpoints(this WebApplication app)
    {
        RouteGroupBuilder routeGroupBuilder = app.MapGroup("/bff");
        routeGroupBuilder.MapGet("/login", Login).AllowAnonymous();
        routeGroupBuilder.MapGet("/logout", Logout).AllowAnonymous();
        routeGroupBuilder.MapGet("/user", User).AllowAnonymous();
        
        return app;
    }

    private static async Task Login(HttpContext httpContext)
    {
        string? returnUrl = httpContext.Request.Query["returnUrl"].FirstOrDefault();

        if (string.IsNullOrEmpty(returnUrl))
        {
            returnUrl = "/";
        }

        AuthenticationProperties properties = new()
        {
            RedirectUri = returnUrl
        };

        await httpContext.ChallengeAsync(properties);
    }

    private static async Task Logout(HttpContext httpContext, IAuthenticationSchemeProvider authenticationSchemeProvider)
    {
        // Remove the cookie first before the sign out
        AuthenticationScheme? signInScheme = await authenticationSchemeProvider.GetDefaultSignInSchemeAsync();
        await httpContext.SignOutAsync(signInScheme?.Name);
        
        string? returnUrl = httpContext.Request.Query["returnUrl"].FirstOrDefault();
        if (string.IsNullOrEmpty(returnUrl))
        {
            if (httpContext.Request.PathBase.HasValue)
            {
                returnUrl = httpContext.Request.PathBase;
            }
            else
            {
                returnUrl = "/";
            }
        }
        
        AuthenticationProperties properties = new()
        {
            RedirectUri = returnUrl
        };

        await httpContext.SignOutAsync(properties);
    }

    private static async Task User(HttpContext httpContext)
    {
        AuthenticateResult result = await httpContext.AuthenticateAsync();

        if (!result.Succeeded)
        {
            httpContext.Response.StatusCode = StatusCodes.Status401Unauthorized;
        }
        else
        {
            List<Claim> claims = [];
            claims.AddRange(GetUserClaims(result));
            claims.AddRange(GetManagementClaims(httpContext, result));

            httpContext.Response.StatusCode = StatusCodes.Status200OK;
            httpContext.Response.ContentType = "application/json";
            await httpContext.Response.WriteAsJsonAsync(claims);
        }
    }

    private static IEnumerable<Claim> GetUserClaims(AuthenticateResult result)
    {
        return result.Principal?.Claims.Select(c => new Claim(c.Type, c.Value)) ?? Enumerable.Empty<Claim>();
    }

    private static IEnumerable<Claim> GetManagementClaims(HttpContext httpContext, AuthenticateResult result)
    {
        List<Claim> claims = [];
        PathString pathBase = httpContext.Request.PathBase;

        string? sessionId = result.Principal?.FindFirst("sid")?.Value;
        if (!string.IsNullOrEmpty(sessionId))
        {
            claims.Add(new Claim("logout-url", $"/bff/logout?sid={sessionId}"));
        }

        if (result.Properties != null)
        {
            if (result.Properties.ExpiresUtc.HasValue)
            {
                double expireInSeconds = result.Properties.ExpiresUtc.Value.Subtract(DateTimeOffset.UtcNow).TotalSeconds;
                claims.Add(new Claim("session-expires-in", Math.Round(expireInSeconds)));
            }

            if (result.Properties.Items.TryGetValue(OpenIdConnectSessionProperties.SessionState, out string? sessionState) && sessionState is not null)
            {
                claims.Add(new Claim("session-state", sessionState));
            }
        }

        return claims;
    }

    private record Claim(string Type, object Value);
}