using System.Net.Http.Headers;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Authentication.OpenIdConnect;
using Microsoft.AspNetCore.HttpOverrides;
using Microsoft.IdentityModel.Protocols.OpenIdConnect;
using ReactAndOpenApi.Configuration;
using ReactAndOpenApi.Endpoints;
using ReactAndOpenApi.Security;
using Yarp.ReverseProxy.Transforms;

var builder = WebApplication.CreateBuilder(args);
{
    builder.Services.AddHttpContextAccessor();
    builder.Services.Configure<TridionOptions>(builder.Configuration.GetSection(TridionOptions.ConfigurationName));
    builder.Services.Configure<ForwardedHeadersOptions>(options =>
    {
        options.ForwardedHeaders = ForwardedHeaders.All;
    });

    // Reverse Proxy
    builder.Services.AddReverseProxy()
        .LoadFromConfig(builder.Configuration.GetSection("ReverseProxy"))
        .AddTransforms(transformBuilderContext =>
        {
            transformBuilderContext.AddRequestTransform(async rtc =>
            {
                string? accessToken = await rtc.HttpContext.GetTokenAsync("access_token");
                if (!string.IsNullOrEmpty(accessToken))
                {
                    rtc.ProxyRequest.Headers.Authorization = new AuthenticationHeaderValue("Bearer", accessToken);
                }
            });
        });
    
    // Get the authority, client id, and cookie expire timespan from the configuration
    TridionOptions tridionOptions = builder.Configuration.GetSection(TridionOptions.ConfigurationName).Get<TridionOptions>()!;
    
    // Authentication
    builder.Services.AddAuthentication(options =>
        {
            options.DefaultScheme = CookieAuthenticationDefaults.AuthenticationScheme;
            options.DefaultChallengeScheme = OpenIdConnectDefaults.AuthenticationScheme;
            options.DefaultSignOutScheme = OpenIdConnectDefaults.AuthenticationScheme;
        })
        .AddCookie(CookieAuthenticationDefaults.AuthenticationScheme, options =>
        {
            options.ExpireTimeSpan = tridionOptions.Authentication.CookieExpireTimeSpan;
            options.SlidingExpiration = false;
            options.Cookie.Name = "__Host-demo";
            options.Cookie.SameSite = SameSiteMode.Strict;
            
            options.SessionStore = new MemoryCacheSessionStore(TimeSpan.FromMinutes(55));
        })
        .AddOpenIdConnect(OpenIdConnectDefaults.AuthenticationScheme, options =>
        {
            options.Authority = tridionOptions.Authentication.Authority;
            options.ClientId = tridionOptions.Authentication.ClientId;
            options.ResponseType = OpenIdConnectResponseType.Code;
            options.ResponseMode = OpenIdConnectResponseMode.Query;
            options.UsePkce = true;
    
            options.MapInboundClaims = false;
            options.GetClaimsFromUserInfoEndpoint = true;
            options.SaveTokens = true;
        
            options.Scope.Clear();
            options.Scope.Add("openid");
            options.Scope.Add("profile");
            options.Scope.Add("offline_access");
    
            options.TokenValidationParameters.NameClaimType = "name";
            options.TokenValidationParameters.RoleClaimType = "role";
    
            options.TokenValidationParameters.ValidateIssuer = true;
            options.TokenValidationParameters.ValidateAudience = true;
        });
}

var app = builder.Build();
{
    // Configure the HTTP request pipeline.
    app.UseForwardedHeaders();
    if (!app.Environment.IsDevelopment())
    {
        // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
        app.UseHsts();
    }

    app.UseHttpsRedirection();

    app.UseAuthentication();
    
    app.UseStaticFiles();
    
    app.MapBackendForFrontendEndpoints();
    app.MapReverseProxy();
    
    app.MapFallbackToFile("index.html");;
}

app.Run();
